// #ifdef H5
import wx from 'weixin-js-sdk'
// #endif
import store from '@/store/index.js'
import {
  isWeixin,
  getQueryVariable
} from "./common.js"
import {
  delayPerform
} from "js-perform-lock";

import md5 from "md5"
import {
  object
} from '@dcloudio/vue-cli-plugin-uni/packages/postcss/tags';

export const navToPage = (function() {
  var gopageid = null
  return function(p, goToType = "navigateTo") {
    var {
      url,
      ...data
    } = p
    // #ifdef H5
    if (isWeixin()) {
      wx.miniProgram.getEnv(res => {
        if (res.miniprogram) {
          var i = 0;
          (function goTo() {
            if (gopageid) {
              clearTimeout(gopageid)
            }
            i++
            if (uniWebview && uniWebview[goToType]) {
              // h5跳转小程序
              if (uni.getStorageSync('miniProgram') == "jsm_env") {
                // 家盛茂小程序环境
                uniWebview[goToType]({
                  url: url,
                  ...data
                })
              } else {
                // 原生小程序页面
                var miniProgramPages = []
                if (!!~miniProgramPages.indexOf(url.split("?")[0])) {
                  // 第三方小程序跳转我们的h5页面处理
                  uniWebview[goToType]({
                    url: `/pagesJsm/pages/middle?webUrl=${encodeURIComponent(webViewHref(url))}`,
                    ...data
                  })
                } else {
                  // 第三方小程序跳转我们的h5页面处理
                  uniWebview[goToType]({
                    url: `/pagesJsm/pages/webview?webUrl=${encodeURIComponent(webViewHref(url))}`,
                    ...data
                  })
                }
              }
            } else if (i < 10) {
              gopageid = setTimeout(function() {
                gopageid = null
                goTo()
              }, 200)
            }
          })()
        } else {
          // 当前环境跳转
          uni[goToType]({
            url: url,
            ...data
          })
        }
      })
    } else {
      // 当前环境跳转
      uni[goToType]({
        url: url,
        ...data
      })
    }
    // #endif
    // #ifndef H5
    // 当前环境跳转
    uni[goToType]({
      url: url,
      ...data
    })
    // #endif
  }
})()

export function webViewHref(url, pam = {}, crossPagePam = {}) {
  var defaultPam = {
    ...pam,
    ...crossPagePam,
    "x-token": store.state.user.token,
    openId: store.state.user.openId,
    userId: store.state.user.userId,
    username: store.state.user.name,
    useravatar: store.state.user.avatar,
    // #ifndef H5
    miniProgram: "jsm_env",
    // #endif
  }

  var pamstr = Object.entries(defaultPam).map(item => item.join("=")).join("&");

  if (!!~url.indexOf("?")) {
    return process.env.VUE_APP_HREF + url + `${pamstr?"&"+pamstr:""}`
  } else {
    return process.env.VUE_APP_HREF + url + `${pamstr?"?"+pamstr:""}`
  }
}

export const crossPage = (function() {
  var funs = {}
  var tiemIdIs = {}
  var datacd = []

  // #ifndef H5
  var delaydeparture = (new delayPerform(250)).refactor(function() {
    var data = {}
    datacd.map(function([name, val]) {
      data[name] = val
    })
    uni.setStorageSync("crossPageTs", encodeURIComponent(JSON.stringify(data)));
    store.commit("user/setcrossPagePam", (new Date().getTime()));
    setTimeout(function() {
      uni.removeStorageSync("crossPageTs")
    }, 350)
    datacd = []
  });
  // #endif

  // #ifdef H5
  var delaydepartFun = (new delayPerform(250)).refactor(function() {
    var data = getQueryVariable("crossPageTs")
    if (data) {
      data = JSON.parse(decodeURIComponent(data))
      Object.keys(data).map(name => {
        Object.keys(funs[name]).map(key => {
          funs[name][key](data[name])
        })
      })
    }
  });
  // #endif

  var funco = {
    $emit(name, data) {
      // 同个环境下可以触发
      if (funs[name]) {
        Object.keys(funs[name]).map(key => {
          funs[name][key](data)
        })
      }

      // #ifndef H5
      // 用于小程序触发更新H5页面
      datacd.push([name, data])
      delaydeparture()
      // #endif

      // #ifdef H5
      // h5在小程序环境下通过定时器传送
      wx.miniProgram.getEnv(res => {
        if (res.miniprogram) {
          var postMessage = (window.uniWebview && window.uniWebview.postMessage) || (window.postMessage)
          if (postMessage) {
            postMessage({
              data: {
                type: "crossPageTs",
                name,
                data
              }
            })
          }
          if (tiemIdIs[name]) {
            clearTimeout(tiemIdIs[name])
          }
          uni.setStorageSync(name, data);
          tiemIdIs[name] = setTimeout(function() {
            uni.removeStorageSync(name)
          }, 350)
        }
      })
      // #endif
    },

    $on(name, fun) {
      if (!funs[name]) {
        funs[name] = {}
      }
      funs[name][md5(fun.toString())] = fun

      // #ifdef H5
      delaydepartFun()
      // #endif
    },

    $off(name, fun) {
      var skey = md5(fun.toString())
      if (funs[name] && funs[name][skey]) {
        delete funs[name][skey]
      }
    },

    // #ifndef H5
    $listener(e) {
      e.detail.data.map(({
        type,
        name,
        data
      }) => {
        if (type == "crossPageTs") {
          if (funs[name]) {
            Object.keys(funs[name]).map(key => {
              funs[name][key](data)
            })
          }
          datacd.push([name, data])
          delaydeparture()
        }
      })
    },
    // #endif
  };

  // #ifdef H5
  wx.miniProgram.getEnv(res => {
    if (res.miniprogram) {
      (function tiemID() {
        Object.keys(funs).map(name => {
          var data = uni.getStorageSync(name);
          if (data) {
            Object.keys(funs[name]).map(key => {
              funs[name][key](data)
            })
          }
        })
        setTimeout(tiemID, 200)
      })()
    }
  })
  // #endif

  return funco
})()